#include <cmath>
#include <numbers>
#include <vector>

#include <filter/filter/low_pass_filter.hpp>

namespace Filter {
LowPassFilter::LowPassFilter(std::size_t size, std::size_t cutoffHz, std::size_t sampleRateHz, const std::vector<float>& window) : IFilter(size) {
  for (std::size_t i = 0; i < size; i++) {
    std::size_t shift = std::round(size / 2);
    float sampleTime = 1.0f / sampleRateHz;
    if (shift != i) {
      IFilter::impulseResponse[i] = std::sin(2.0f * std::numbers::pi * sampleTime * (i - shift)) / (std::numbers::pi * sampleTime * (i - shift));
    } else {
      IFilter::impulseResponse[i] = 2.0f * cutoffHz;
    }

    IFilter::impulseResponse[i] *= sampleTime * window[i];
  }
}

} // namespace Filter