#include <cstdio>
#include <cstring>
#include <iostream>
#include <vector>

#include <filter/filter/low_pass_filter.hpp>
#include <filter/window/hamming_window.hpp>

#include <c-flags.h>

const std::size_t BUFFER_SIZE = 256;

using namespace Filter;

int main(int argc, char** argv) {
  (void)argc;
  (void)argv;

  // prepare pipes for binary data
  std::freopen(nullptr, "rb", stdin);
  if (std::ferror(stdin)) {
    std::cerr << "Failed to reopen stdin: " << std::strerror(errno) << std::endl;
    return errno;
  }

  std::freopen(nullptr, "wb", stdout);
  if (std::ferror(stdout)) {
    std::cerr << "Failed to reopen stdout: " << std::strerror(errno) << std::endl;
    return errno;
  }

  // create low pass filter
  LowPassFilter lpf(BUFFER_SIZE, 1000, 10000, HammingWindow(BUFFER_SIZE).getWindow());

  // read data
  std::vector<float> inputBuffer;
  inputBuffer.reserve(BUFFER_SIZE);

  std::vector<float> outputBuffer;
  outputBuffer.reserve(BUFFER_SIZE);

  std::size_t len;

  while ((len = std::fread(inputBuffer.data(), sizeof(inputBuffer[0]), BUFFER_SIZE, stdin)) > 0) {
    if (std::ferror(stdin) && !std::feof(stdin)) {
      std::cerr << "Failed to read from stdin: " << std::strerror(errno) << std::endl;
      return errno;
    }

    lpf.ConvolveIQ(inputBuffer, outputBuffer);
  }

  return 0;
}